# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: discount/grpc/discount.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x64iscount/grpc/discount.proto\x12\x10\x64\x61rsman.discount\x1a\x1cgoogle/protobuf/struct.proto\"=\n\rBasketRequest\x12,\n\x05items\x18\x01 \x03(\x0b\x32\x1d.darsman.discount.ItemRequest\"v\n\rCouponRequest\x12\x13\n\x0b\x63oupon_code\x18\x01 \x01(\t\x12,\n\x05items\x18\x02 \x03(\x0b\x32\x1d.darsman.discount.ItemRequest\x12\x15\n\x08owner_id\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_owner_id\"S\n\x18InstallmentBasketRequest\x12\x37\n\x05items\x18\x01 \x03(\x0b\x32(.darsman.discount.InstallmentItemRequest\"x\n\x16InstallmentItemRequest\x12\x0f\n\x07item_id\x18\x01 \x01(\t\x12\x11\n\titem_type\x18\x02 \x01(\t\x12\x16\n\x0eupfront_amount\x18\x03 \x01(\x05\x12\x13\n\x0bpart_amount\x18\x04 \x01(\x05\x12\r\n\x05parts\x18\x05 \x01(\x05\"@\n\x0bItemRequest\x12\x0f\n\x07item_id\x18\x01 \x01(\t\x12\x11\n\titem_type\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\x05\"o\n\x08Response\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"\'\n\x15TieredDiscountRequest\x12\x0e\n\x06\x61mount\x18\x01 \x01(\x05\"i\n\x0cUsageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0f\x64iscount_amount\x18\x02 \x01(\x05\x12\x13\n\x06\x63oupon\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07\x62\x61skets\x18\x04 \x03(\tB\t\n\x07_coupon2\xd0\x03\n\x12\x44iscountController\x12\x63\n\x17\x43\x61lcInstallmentByBasket\x12*.darsman.discount.InstallmentBasketRequest\x1a\x1a.darsman.discount.Response\"\x00\x12R\n\x11\x43\x61lculateByBasket\x12\x1f.darsman.discount.BasketRequest\x1a\x1a.darsman.discount.Response\"\x00\x12R\n\x11\x43\x61lculateByCoupon\x12\x1f.darsman.discount.CouponRequest\x1a\x1a.darsman.discount.Response\"\x00\x12`\n\x17\x43\x61lculateTieredDiscount\x12\'.darsman.discount.TieredDiscountRequest\x1a\x1a.darsman.discount.Response\"\x00\x12K\n\x0b\x43reateUsage\x12\x1e.darsman.discount.UsageRequest\x1a\x1a.darsman.discount.Response\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'discount.grpc.discount_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_BASKETREQUEST']._serialized_start=80
  _globals['_BASKETREQUEST']._serialized_end=141
  _globals['_COUPONREQUEST']._serialized_start=143
  _globals['_COUPONREQUEST']._serialized_end=261
  _globals['_INSTALLMENTBASKETREQUEST']._serialized_start=263
  _globals['_INSTALLMENTBASKETREQUEST']._serialized_end=346
  _globals['_INSTALLMENTITEMREQUEST']._serialized_start=348
  _globals['_INSTALLMENTITEMREQUEST']._serialized_end=468
  _globals['_ITEMREQUEST']._serialized_start=470
  _globals['_ITEMREQUEST']._serialized_end=534
  _globals['_RESPONSE']._serialized_start=536
  _globals['_RESPONSE']._serialized_end=647
  _globals['_TIEREDDISCOUNTREQUEST']._serialized_start=649
  _globals['_TIEREDDISCOUNTREQUEST']._serialized_end=688
  _globals['_USAGEREQUEST']._serialized_start=690
  _globals['_USAGEREQUEST']._serialized_end=795
  _globals['_DISCOUNTCONTROLLER']._serialized_start=798
  _globals['_DISCOUNTCONTROLLER']._serialized_end=1262
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import discount_pb2 as discount_dot_grpc_dot_discount__pb2


class DiscountControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CalcInstallmentByBasket = channel.unary_unary(
                '/darsman.discount.DiscountController/CalcInstallmentByBasket',
                request_serializer=discount_dot_grpc_dot_discount__pb2.InstallmentBasketRequest.SerializeToString,
                response_deserializer=discount_dot_grpc_dot_discount__pb2.Response.FromString,
                )
        self.CalculateByBasket = channel.unary_unary(
                '/darsman.discount.DiscountController/CalculateByBasket',
                request_serializer=discount_dot_grpc_dot_discount__pb2.BasketRequest.SerializeToString,
                response_deserializer=discount_dot_grpc_dot_discount__pb2.Response.FromString,
                )
        self.CalculateByCoupon = channel.unary_unary(
                '/darsman.discount.DiscountController/CalculateByCoupon',
                request_serializer=discount_dot_grpc_dot_discount__pb2.CouponRequest.SerializeToString,
                response_deserializer=discount_dot_grpc_dot_discount__pb2.Response.FromString,
                )
        self.CalculateTieredDiscount = channel.unary_unary(
                '/darsman.discount.DiscountController/CalculateTieredDiscount',
                request_serializer=discount_dot_grpc_dot_discount__pb2.TieredDiscountRequest.SerializeToString,
                response_deserializer=discount_dot_grpc_dot_discount__pb2.Response.FromString,
                )
        self.CreateUsage = channel.unary_unary(
                '/darsman.discount.DiscountController/CreateUsage',
                request_serializer=discount_dot_grpc_dot_discount__pb2.UsageRequest.SerializeToString,
                response_deserializer=discount_dot_grpc_dot_discount__pb2.Response.FromString,
                )


class DiscountControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CalcInstallmentByBasket(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CalculateByBasket(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CalculateByCoupon(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CalculateTieredDiscount(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateUsage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DiscountControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CalcInstallmentByBasket': grpc.unary_unary_rpc_method_handler(
                    servicer.CalcInstallmentByBasket,
                    request_deserializer=discount_dot_grpc_dot_discount__pb2.InstallmentBasketRequest.FromString,
                    response_serializer=discount_dot_grpc_dot_discount__pb2.Response.SerializeToString,
            ),
            'CalculateByBasket': grpc.unary_unary_rpc_method_handler(
                    servicer.CalculateByBasket,
                    request_deserializer=discount_dot_grpc_dot_discount__pb2.BasketRequest.FromString,
                    response_serializer=discount_dot_grpc_dot_discount__pb2.Response.SerializeToString,
            ),
            'CalculateByCoupon': grpc.unary_unary_rpc_method_handler(
                    servicer.CalculateByCoupon,
                    request_deserializer=discount_dot_grpc_dot_discount__pb2.CouponRequest.FromString,
                    response_serializer=discount_dot_grpc_dot_discount__pb2.Response.SerializeToString,
            ),
            'CalculateTieredDiscount': grpc.unary_unary_rpc_method_handler(
                    servicer.CalculateTieredDiscount,
                    request_deserializer=discount_dot_grpc_dot_discount__pb2.TieredDiscountRequest.FromString,
                    response_serializer=discount_dot_grpc_dot_discount__pb2.Response.SerializeToString,
            ),
            'CreateUsage': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateUsage,
                    request_deserializer=discount_dot_grpc_dot_discount__pb2.UsageRequest.FromString,
                    response_serializer=discount_dot_grpc_dot_discount__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.discount.DiscountController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DiscountController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CalcInstallmentByBasket(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.discount.DiscountController/CalcInstallmentByBasket',
            discount_dot_grpc_dot_discount__pb2.InstallmentBasketRequest.SerializeToString,
            discount_dot_grpc_dot_discount__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CalculateByBasket(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.discount.DiscountController/CalculateByBasket',
            discount_dot_grpc_dot_discount__pb2.BasketRequest.SerializeToString,
            discount_dot_grpc_dot_discount__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CalculateByCoupon(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.discount.DiscountController/CalculateByCoupon',
            discount_dot_grpc_dot_discount__pb2.CouponRequest.SerializeToString,
            discount_dot_grpc_dot_discount__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CalculateTieredDiscount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.discount.DiscountController/CalculateTieredDiscount',
            discount_dot_grpc_dot_discount__pb2.TieredDiscountRequest.SerializeToString,
            discount_dot_grpc_dot_discount__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateUsage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.discount.DiscountController/CreateUsage',
            discount_dot_grpc_dot_discount__pb2.UsageRequest.SerializeToString,
            discount_dot_grpc_dot_discount__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import ClassVar, List, Optional, Type, TypeVar
from uuid import UUID

from google.protobuf.json_format import MessageToDict
from google.protobuf.message import Message
from pydantic import BaseModel, validate_call

from dmdc.grpc.discount_pb2 import InstallmentItemRequest, ItemRequest
from dmdc.schemas import (
    CouponCalculateResult,
    DiscountedInstallmentItems,
    DiscountedItems,
    InstallmentItem,
    Item,
    TieredDiscountResult,
)

from . import config
from .api_constants import (
    SUCCESS_CODE,
    RequestMessages,
    RPCMethodNames,
    Stubs,
)

M = TypeVar("M", bound=BaseModel)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    _instances: ClassVar[dict] = {}
    stub_cls: ClassVar

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    @validate_call(config=dict(arbitrary_types_allowed=True))
    async def perform_rpc(
        self,
        *,
        rpc_method_name: str,
        request_message: Optional[Message] = None,
        response_model: Optional[Type[M]] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ) -> list[M] | dict:
        """Perform the remote-precedure-call.

        Parameters:
            rpc_method_name: The name of the RPC method (e.g., `Retrieve`).
            request_message: The request message instance.
            timeout: RPC timeout.
            wait_for_ready: Wait for Channel to be ready in case is not.
        """
        if config.CHANNEL is None:
            await config.construct_channel()

        RPCMethod = getattr(self.stub_cls(config.CHANNEL), rpc_method_name, None)
        if not RPCMethod:
            raise ValidationError("Invalid RPC method name.")

        metadata = [("api-key", config.grpc_settings.API_KEY)]
        if request_id:
            metadata.append(("x-request-id", str(request_id)))

        response = await RPCMethod(
            request_message,
            metadata=metadata,
            timeout=config.grpc_settings.TIMEOUT if timeout is None else timeout,
            wait_for_ready=config.grpc_settings.WAIT_FOR_READY
            if wait_for_ready is None
            else wait_for_ready,
        )

        # Raise error if response is not successful
        if response.code != SUCCESS_CODE:
            raise ResponseCodeError(response.code)

        response_dict = MessageToDict(response, preserving_proto_field_name=True)
        data = response_dict.get("data", [])

        if not response_model:
            return data

        return [response_model.model_validate(item) for item in data]


class Discount(RPCClient):
    stub_cls = Stubs.DISCOUNT

    @validate_call
    async def calculate_by_coupon(
        self,
        coupon_code: str,
        items: List[Item],
        owner_id: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Calculate and return detailed discount data for provided coupon."""

        # Assign attributes
        request = RequestMessages.CALCULATE_BY_COUPON(
            coupon_code=coupon_code,
            owner_id=owner_id,
            items=[ItemRequest(**item.model_dump()) for item in items],
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CALCULATE_BY_COUPON,
            request_message=request,
            response_model=CouponCalculateResult,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def calculate_by_basket(
        self,
        items: List[Item],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Calculate and return discount data for provided items if they're in active baskets."""

        # Assign attributes
        request = RequestMessages.CALCULATE_BY_BASKET(
            items=[ItemRequest(**item.model_dump()) for item in items]
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CALCULATE_BY_BASKET,
            request_message=request,
            response_model=DiscountedItems,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def calculate_installment_by_basket(
        self,
        items: List[InstallmentItem],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Calculate and return discount data for provided installment items if they're in active baskets."""

        # Assign attributes
        request = RequestMessages.CALCULATE_INSTALLMENT_BY_BASKET(
            items=[InstallmentItemRequest(**item.model_dump()) for item in items]
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CALCULATE_INSTALLMENT_BY_BASKET,
            request_message=request,
            response_model=DiscountedInstallmentItems,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def calculate_tiered_discount(
        self,
        amount: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Calculate and return tiered discount data for the provided amount."""

        request = RequestMessages.CALCULATE_TIERED_DISCOUNT(amount=amount)

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CALCULATE_TIERED_DISCOUNT,
            request_message=request,
            response_model=TieredDiscountResult,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def create_usage(
        self,
        user_id: str,
        discount_amount: int,
        coupon: Optional[str] = None,
        baskets: Optional[List[str]] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Create discount usage for a user with optional coupon and baskets."""

        baskets = baskets or []

        # build the UsageRequest
        request = RequestMessages.CREATE_USAGE(
            user_id=user_id,
            discount_amount=discount_amount,
            coupon=coupon,
            baskets=baskets,
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CREATE_USAGE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )



from .grpc.discount_pb2 import (
    BasketRequest,
    CouponRequest,
    InstallmentBasketRequest,
    TieredDiscountRequest,
    UsageRequest,
)
from .grpc.discount_pb2_grpc import DiscountControllerStub

SUCCESS_CODE = "0"


def _singletonize_class_vars(cls):
    for attr_name, attr_value in cls.__dict__.items():
        if isinstance(attr_value, type):  # it's a class
            orig_cls = attr_value

            class SingletonWrapper(orig_cls):
                _instances = {}

                def __new__(cls, *args, **kwargs):
                    if cls not in cls._instances:
                        cls._instances[cls] = super().__new__(cls)
                    return cls._instances[cls]

            setattr(cls, attr_name, SingletonWrapper)
    return cls


@_singletonize_class_vars
class Stubs:
    DISCOUNT = DiscountControllerStub


class RequestMessages:
    CALCULATE_BY_COUPON = CouponRequest
    CALCULATE_BY_BASKET = BasketRequest
    CALCULATE_INSTALLMENT_BY_BASKET = InstallmentBasketRequest
    CALCULATE_TIERED_DISCOUNT = TieredDiscountRequest
    CREATE_USAGE = UsageRequest


class RPCMethodNames:
    CALCULATE_BY_COUPON = "CalculateByCoupon"
    CALCULATE_BY_BASKET = "CalculateByBasket"
    CALCULATE_INSTALLMENT_BY_BASKET = "CalcInstallmentByBasket"
    CALCULATE_TIERED_DISCOUNT = "CalculateTieredDiscount"
    CREATE_USAGE = "CreateUsage"



from typing import List, Optional

from pydantic import BaseModel


# Base
class BaseItem(BaseModel):
    item_id: str
    item_type: str


class BaseDiscountedItem(BaseModel):
    discount: float
    applied_percentage: float
    discount_identifier: Optional[str]


# Request
class Item(BaseItem):
    price: int


class InstallmentItem(BaseItem):
    upfront_amount: int
    part_amount: int
    parts: int


# Response
class DiscountedItem(Item, BaseDiscountedItem):
    discounted_price: float


class DiscountedItems(BaseModel):
    items: List[DiscountedItem]


class DiscountedInstallmentItem(InstallmentItem, BaseDiscountedItem):
    discounted_upfront_amount: float
    discounted_part_amount: float
    total_discounted_amount: float


class DiscountedInstallmentItems(BaseModel):
    items: List[DiscountedInstallmentItem]


class CouponCalculateResult(BaseModel):
    coupon_percentage: float
    items: List[DiscountedItem]
    total_discount: float


class TieredDiscountResult(BaseModel):
    amount: int
    discount_amount: float
    discounted_amount: float
